@@ -33,6 +33,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_json_lib.h>
 #include <gnunet/gnunet_mhd_compat.h>
+#include <gnunet/gnunet_mhd_lib.h>
 #include "twister.h"
 #include "taler_twister_service.h"
 #include <jansson.h>
@@ -640,7 +641,8 @@ curl_upload_cb (void *buf,
    * We got rescheduled because the download callback was asleep.
    * FIXME: can this block be eliminated and the unpausing being
    * moved in the last block where we return zero as well?
-   */if ( (0 == hr->io_len) &&
+   */
+  if ( (0 == hr->io_len) &&
        (REQUEST_STATE_PROXY_DOWNLOAD_STARTED == hr->state) )
   {
     if (GNUNET_YES == hr->curl_paused)
@@ -1324,7 +1326,8 @@ inflate_data (struct HttpRequest *request)
   memset (&z, 0, sizeof (z));
   z.next_in = (Bytef *) request->io_buf;
   z.avail_in = request->io_len;
-  tmp_size = GNUNET_MIN (REQUEST_BUFFER_MAX, request->io_len * 4);
+  tmp_size = GNUNET_MIN (REQUEST_BUFFER_MAX,
+                         request->io_len * 4);
   tmp = GNUNET_malloc (tmp_size);
   z.next_out = (Bytef *) tmp;
   z.avail_out = tmp_size;
@@ -1333,10 +1336,10 @@ inflate_data (struct HttpRequest *request)
   {
   case Z_MEM_ERROR:
     GNUNET_break (0);
-    return GNUNET_JSON_PR_OUT_OF_MEMORY;
+    return GNUNET_MHD_PR_OUT_OF_MEMORY;
   case Z_STREAM_ERROR:
     GNUNET_break_op (0);
-    return GNUNET_JSON_PR_JSON_INVALID;
+    return GNUNET_MHD_PR_JSON_INVALID;
   case Z_OK:
     break;
   }
@@ -1349,17 +1352,17 @@ inflate_data (struct HttpRequest *request)
       GNUNET_break (0);
       GNUNET_break (Z_OK == inflateEnd (&z));
       GNUNET_free (tmp);
-      return GNUNET_JSON_PR_OUT_OF_MEMORY;
+      return GNUNET_MHD_PR_OUT_OF_MEMORY;
     case Z_DATA_ERROR:
       GNUNET_break (0);
       GNUNET_break (Z_OK == inflateEnd (&z));
       GNUNET_free (tmp);
-      return GNUNET_JSON_PR_JSON_INVALID;
+      return GNUNET_MHD_PR_JSON_INVALID;
     case Z_NEED_DICT:
       GNUNET_break (0);
       GNUNET_break (Z_OK == inflateEnd (&z));
       GNUNET_free (tmp);
-      return GNUNET_JSON_PR_JSON_INVALID;
+      return GNUNET_MHD_PR_JSON_INVALID;
     case Z_OK:
       if ((0 < z.avail_out) && (0 == z.avail_in))
       {
@@ -1367,7 +1370,7 @@ inflate_data (struct HttpRequest *request)
         GNUNET_break (0);
         GNUNET_break (Z_OK == inflateEnd (&z));
         GNUNET_free (tmp);
-        return GNUNET_JSON_PR_JSON_INVALID;
+        return GNUNET_MHD_PR_JSON_INVALID;
       }
       if (0 < z.avail_out)
         continue; /* just call it again */
@@ -1378,7 +1381,7 @@ inflate_data (struct HttpRequest *request)
         GNUNET_break (0);
         GNUNET_break (Z_OK == inflateEnd (&z));
         GNUNET_free (tmp);
-        return GNUNET_JSON_PR_OUT_OF_MEMORY;
+        return GNUNET_MHD_PR_OUT_OF_MEMORY;
       }
       if (tmp_size * 2 < tmp_size)
         tmp_size = REQUEST_BUFFER_MAX;
@@ -1394,7 +1397,7 @@ inflate_data (struct HttpRequest *request)
       request->io_size = tmp_size;
       request->io_len = z.total_out;
       GNUNET_break (Z_OK == inflateEnd (&z));
-      return GNUNET_JSON_PR_SUCCESS; /* at least for now */
+      return GNUNET_MHD_PR_SUCCESS; /* at least for now */
     }
   } /* while (1) */
 }
@@ -1580,12 +1583,12 @@ create_response (void *cls,
     /* Malform request body.  Note, this flag will be
      * cleared only after having set the right malformed
      * body length in the request headers.  */
-    if (GNUNET_YES == malform_upload)
+    if ( (GNUNET_YES == malform_upload) &&
+         (hr->io_len > 0) )
     {
-      TWISTER_LOG_DEBUG
-        ("Will (badly) truncate the request.\n");
-      malformed_size = GNUNET_CRYPTO_random_u32
-                         (GNUNET_CRYPTO_QUALITY_WEAK, hr->io_len);
+      TWISTER_LOG_DEBUG ("Will (badly) truncate the request.\n");
+      malformed_size = GNUNET_CRYPTO_random_u32 (GNUNET_CRYPTO_QUALITY_WEAK,
+                                                 hr->io_len);
       hr->io_len = malformed_size;
       malform_upload = GNUNET_NO;
     }
@@ -1905,12 +1908,15 @@ create_response (void *cls,
 
   if (GNUNET_YES == malform)
   {
-    size_t fake_len;
-
     TWISTER_LOG_DEBUG ("Will (badly) truncate the response.\n");
-    fake_len = GNUNET_CRYPTO_random_u32 (GNUNET_CRYPTO_QUALITY_WEAK,
-                                         hr->io_len);
-    hr->io_len = fake_len;
+    if (hr->io_len > 0)
+    {
+      size_t fake_len;
+
+      fake_len = GNUNET_CRYPTO_random_u32 (GNUNET_CRYPTO_QUALITY_WEAK,
+                                           hr->io_len);
+      hr->io_len = fake_len;
+    }
     malform = GNUNET_NO;
   }
 
